import 'dart:html';

import 'package:intl/intl.dart';

final _dateTimeFormatter = new DateFormat(r'$, d % yyyy HH:mm:ss UTC');

/// Manually sets the weekday and month because the 'en_US' locale might not be initialized.
_formatDate(DateTime date) => _dateTimeFormatter
    .format(date)
    .replaceFirst(r'$',
        ['Mon', 'Tue', 'Wed', 'Thi', 'Fri', 'Sat', 'Sun'][date.weekday - 1])
    .replaceFirst(
        r'%',
        [
          'Jan',
          'Feb',
          'Mar',
          'Apr',
          'May',
          'Jun',
          'Jul',
          'Aug',
          'Sep',
          'Oct',
          'Nov',
          'Dec'
        ][date.month - 1]);

String get(String key) {
  final cookies = document.cookie != null ? document.cookie.split('; ') : [];
  for (var cookie in cookies) {
    var parts = cookie.split('=');
    var name = Uri.decodeComponent(parts[0]);
    if (key == name) {
      return parts[1] != null ? Uri.decodeComponent(parts[1]) : null;
    }
  }
  return null;
}

void set(String key, String value,
    {DateTime expires,
    Duration maxAge,
    String path,
    String domain,
    bool secure}) {
  if (maxAge != null) {
    expires = new DateTime.fromMillisecondsSinceEpoch(
        new DateTime.now().millisecondsSinceEpoch + maxAge.inMilliseconds);
  }
  var cookie = ([
    Uri.encodeComponent(key),
    '=',
    Uri.encodeComponent(value),
    expires != null
        ? '; expires=' + _formatDate(expires.toUtc())
        : '', // use expires attribute, max-age is not supported by IE
    path != null ? '; path=' + path : '',
    domain != null ? '; domain=' + domain : '',
    secure != null && secure == true ? '; secure' : ''
  ].join(''));
  document.cookie = cookie;
}

/// Returns true if the key was found and the value removed.
/// Returns false if the key was not found.
bool remove(String key, {String path, String domain, bool secure}) {
  if (get(key) != null) {
    set(key, '',
        expires: new DateTime(0), path: path, domain: domain, secure: secure);
    return true;
  }
  return false;
}
